module cipher(in_data, clk, load_d, sub_key_in, mode, data_out, round, done);

/***********************INPUTS*******************************/
input [63:0] in_data;    // 64 bit plain text input
input [47:0] sub_key_in; // Rounds Subkeys input
input clk;               // Clock input
input load_d;            // Loads the data
input mode;              // mode = 0 or 1 for encrpytion or decryption respectively

/**********************OUTPUTS******************************/
output [63:0] data_out;   // 64 bit cipher text output 
output [3:0] round;       // Round number used for subkey selection
output done;           // Indicates that rounds are completed

/*********************REGISTERED OUTPUTS********************/
reg [3:0] round;
reg done;

/*********************LOCAL VARIABLES***********************/
reg [31:0] left, right;
reg [3:0] index;
wire [31:0] b, rightbar, pout;
wire [47:0] xor1, expout;
 
always@(posedge clk or negedge reset)
  begin
  if(!reset)
    begin
    round <= 5'bzzzzz;
    end
  else
    begin
    if(load_d)
      begin
      left <={indata[6],indata[14],indata[22],indata[30],indata[38],
               indata[46],indata[54],indata[62],indata[4],indata[12],
               indata[20],indata[28],indata[36],indata[44],indata[52],
               indata[60],indata[2],indata[10],indata[18],indata[26],
               indata[34],indata[42],indata[50],indata[58],indata[0],
               indata[8],indata[16],indata[24],indata[32],indata[40],
               indata[48],indata[56]};

      right <={indata[7],indata[15],indata[23],indata[31],indata[39],
                indata[47],indata[55],indata[63],indata[5],indata[13],
                indata[21],indata[29],indata[37],indata[45],indata[53],
                indata[61],indata[3],indata[11],indata[19],indata[27],
                indata[35],indata[43],indata[51],indata[59],indata[1],
                indata[9],indata[17],indata[25],indata[33],indata[41],
                indata[49],indata[57]};  
      if(!mode)
        round <= 0;
      else
        round <= 15;
      index <= 0;                                        
      end 
   
    else if(iterate)
      begin
      left <= right;
      right <= rightbar;
      if(!mode)
        round <= round + 1;
      else
        round <= round - 1;
      index <= index + 1;
      end
    else
      round <= 5'bzzzzz;
    end
  end
 
assign done = (index<15) ? 0 : 1;
  
expermute ep(right, expout); 
 
 assign xor1 = expout ^ subkey;
 
 s1box s1(b[3:0], xor1[5:0]);
 s2box s2(b[7:4], xor1[11:6]);
 s3box s3(b[11:8],xor1[17:12]);
 s4box s4(b[15:12],xor1[23:18]);
 s5box s5(b[19:16],xor1[29:24]);
 s6box s6(b[23:20],xor1[35:30]);
 s7box s7(b[27:24],xor1[41:36]);
 s8box s8(b[31:28],xor1[47:42]);

 permuteb pb(b, pout);
 
 assign rightbar = pout ^ left;

 fpermute fp(dout,{left,right});
 
endmodule
