module syn_ram_16x48_irou (Q, Data, WE, Address, Inclock);

parameter LPM_FILE = "UNUSED";
parameter Width = 48;
parameter WidthAd = 4;
parameter NumWords = 16;

input [WidthAd-1:0] Address;
input [Width-1:0] Data;
input WE;
output [Width-1:0] Q;
input Inclock;


// internal reg
reg [Width-1:0] mem_data [NumWords-1:0];
reg [Width-1:0] tmp_q, tmp_q_reg;
reg [Width-1:0] ZERO, UNKNOWN, HiZ;
reg [8*256:1] mem_initf;
reg [WidthAd-1:0] clocked_address;

reg [Width-1:0] clocked_data;

reg clocked_we;

integer i;

function ValidAddress;
    input [WidthAd-1:0] Address;
    begin
        ValidAddress = 1'b0;
        if(^Address ==='bx)
              $display("%d:Error: Invalid address", $time);
        else if(Address >= NumWords)
              $display("%d:Error: Address out of bounds on RAM", $time);
        else
              ValidAddress = 1'b1;
    end
endfunction

initial
begin

  // check for number of words out of bound
   if(NumWords > (1 << WidthAd))
         $display("Error: Too many words defined");

   for (i=0; i < Width; i=i+1)
      begin
          ZERO[i] = 1'b0;
          UNKNOWN[i] = 1'bX;
      end

   for(i = 0; i < NumWords; i=i+1)
         mem_data[i] = ZERO;

  // load data to the RAM
   if (LPM_FILE != "UNUSED")
      begin
   //        $convert_hex2ver(LPM_FILE, Width, mem_initf);
           $readmemh(mem_initf, mem_data);
      end
end

initial
begin

   clocked_data = 0;
   clocked_address = 0;
   clocked_we = 0;
   tmp_q = 0;

end

  always @( posedge Inclock)
     begin
        clocked_address <= Address;
        clocked_data <= Data;
        clocked_we <= WE;
     end 
 
  always @( negedge Inclock )  
     begin
        if (clocked_we)  
                mem_data[clocked_address] <= clocked_data;
     end 
 
 
  assign Q = mem_data[clocked_address];


endmodule // syn_ram_16x48_irou

