module Keygen (kin, clk, loadk, kout, round, wr, kready);

/***********************INPUTS*******************************/
input [0:63]    kin;            // 64 bit Key input to the system
input           clk;            // System clock
input           loadk;          // Loads the key

/**********************OUTPUTS******************************/
output [47:0]   kout;           // Sub key output to the memory module    
output          wr;             // Memory write signal
output [4:0]    round;          // Memory address signal
output          kready;         // Indicates completion of subkeys generation

/*********************REGISTERED OUTPUTS********************/
reg   [4:0]     round;
reg             kready;
reg             wr;

/*********************LOCAL VARIABLES***********************/
reg   [27:0]    c, d;
reg   [27:0]    ctemp, dtemp;
wire  [0:55]    temp;

            always@(posedge clk)
            begin              
              if(loadk)
              begin	 
		c <={kin[56],kin[48],kin[40],kin[32],kin[24],kin[16],kin[8],
		 kin[0],kin[57],kin[49],kin[41],kin[33],kin[25],kin[17],
		 kin[9],kin[1],kin[58],kin[50],kin[42],kin[34],kin[26],
		 kin[18],kin[10],kin[2],kin[59],kin[51],kin[43],kin[35] };

		d <={kin[62],kin[54],kin[46],kin[38],kin[30],kin[22],kin[14],
		 kin[6],kin[61],kin[53],kin[45],kin[37],kin[29],kin[21],
		 kin[13],kin[5],kin[60],kin[52],kin[44],kin[36],kin[28],
		 kin[20],kin[12],kin[4],kin[27],kin[19],kin[11],kin[3] }; 	

                round <= 0;
                kready <= 0;
                wr <= 1;
              end
         
              else if(round < 15)
              begin
                c <= ctemp;
                d <= dtemp;
                round <= round+1;
                wr <= 1;
              end

              else
              begin
                c <= ctemp;
                d <= dtemp;
                kready <= 1;
                wr <= 0;
                round <= 5'bz;
              end
            end
         
            always@(round or c or d)
            begin
              case(round) 
              0: begin 
                 ctemp={c[26:0],c[27]};
                 dtemp={d[26:0],d[27]};
                 end

              1: begin
                 ctemp={c[26:0],c[27]};
                 dtemp={d[26:0],d[27]};
                 end

              2: begin
                 ctemp={c[25:0],c[27],c[26]};
                 dtemp={d[25:0],d[27],d[26]};
                 end

              3: begin
                 ctemp={c[25:0],c[27],c[26]};
                 dtemp={d[25:0],d[27],d[26]};
                 end

              4: begin
                 ctemp={c[25:0],c[27],c[26]};
                 dtemp={d[25:0],d[27],d[26]};
                 end

              5: begin
                 ctemp={c[25:0],c[27],c[26]};
                 dtemp={d[25:0],d[27],d[26]};
                 end

              6: begin
                 ctemp={c[25:0],c[27],c[26]};
                 dtemp={d[25:0],d[27],d[26]};
                 end

              7: begin 
                 ctemp={c[25:0],c[27],c[26]};
                 dtemp={d[25:0],d[27],d[26]};
                 end

              8: begin
                 ctemp={c[26:0],c[27]};
                 dtemp={d[26:0],d[27]};
                 end

              9: begin
                 ctemp={c[25:0],c[27],c[26]};
                 dtemp={d[25:0],d[27],d[26]};
                 end

              10: begin   
                  ctemp={c[25:0],c[27],c[26]};
                  dtemp={d[25:0],d[27],d[26]};
                  end

              11: begin  
                  ctemp={c[25:0],c[27],c[26]};
                  dtemp={d[25:0],d[27],d[26]};
                  end

              12: begin
                  ctemp={c[25:0],c[27],c[26]};
                  dtemp={d[25:0],d[27],d[26]};
                  end

              13: begin
                  ctemp={c[25:0],c[27],c[26]};
                  dtemp={d[25:0],d[27],d[26]};
                  end

              14: begin
                  ctemp={c[25:0],c[27],c[26]};
                  dtemp={d[25:0],d[27],d[26]};
                  end

              15: begin  
                  ctemp={c[26:0],c[27]};
                  dtemp={d[26:0],d[27]};
                  end
			  
			  default:
			  	begin
				ctemp = ctemp;
				dtemp = dtemp;
				end
              endcase
          end
              
       assign temp={ctemp,dtemp};  
		 
		 assign kout={temp[13],temp[16],temp[10],temp[23],temp[0],temp[4],                    
                 temp[2],temp[27],temp[14],temp[5],temp[20],temp[9],
                 temp[22],temp[18],temp[11],temp[3],temp[25],temp[7],
                 temp[15],temp[6],temp[26],temp[19],temp[12],temp[1],
                 
                 temp[40],temp[51],temp[30],temp[36],temp[46],temp[54],
                 temp[29],temp[39],temp[50],temp[44],temp[32],temp[47],
                 temp[43],temp[48],temp[38],temp[55],temp[33],temp[52],
                 temp[45],temp[41],temp[49],temp[35],temp[28],temp[31]};		
endmodule
